-------------------------------------------------------------------------------
-- HEIG-VD, Haute Ecole d'Ingenierie et de Gestion du canton de Vaud
-- Institut REDS, Reconfigurable & Embedded Digital Systems
--
-- File         : tictactoe_dual_tb.vhd
-- Description  :
--
-- Author       :
-- Date         : 30.05.17
-- Version      : 0.0
--
-- Dependencies :
--
--| Modifications |------------------------------------------------------------
-- Version   Author Date               Description
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library tlmvm;
context tlmvm.tlmvm_context;

use work.tictactoe_config_pkg.all;

entity tictactoe_dual_tb is
end entity;

architecture testbench of tictactoe_dual_tb is

    ---------------
    -- Constants --
    ---------------
    constant MASTER_CLK_PERIOD : time := 7 ns;
    constant SLAVE_CLK_PERIOD  : time := 11 ns;

    ----------------
    -- Components --
    ----------------
    component tictactoe_top is
        port (
            clk_i        : in  std_logic;
            rst_i        : in  std_logic;
            -- Game controls
            keys_i       : in  std_logic_vector(2 downto 0);
            switchs_i    : in  std_logic_vector(7 downto 0);
            -- Game state
            game_state_o : out game_state_t;
            -- Master interface
            master_en_i  : in  std_logic;
            sclk_o       : out std_logic;
            mosi_o       : out std_logic;
            miso_i       : in  std_logic;
            n_ss_o       : out std_logic;
            -- Slave interface
            sclk_i       : in  std_logic;
            mosi_i       : in  std_logic;
            miso_o       : out std_logic;
            n_ss_i       : in  std_logic);
    end component;

    ----------------
    -- Procedures --
    ----------------
    procedure rep(status : finish_status_t) is
    begin
        report "End of simulation";
    end rep;

    -------------
    -- Signals --
    -------------

    -- Simulation signals
    signal finished_s : std_logic := '0';

    -- Master signals
    signal clk_master_sti        : std_logic;
    signal rst_master_sti        : std_logic;
    signal keys_master_sti       : std_logic_vector(2 downto 0);
    signal switches_master_sti   : std_logic_vector(7 downto 0);
    signal game_state_master_obs : game_state_t;

    -- Slave signals
    signal clk_slave_sti        : std_logic;
    signal rst_slave_sti        : std_logic;
    signal keys_slave_sti       : std_logic_vector(2 downto 0);
    signal switches_slave_sti   : std_logic_vector(7 downto 0);
    signal game_state_slave_obs : game_state_t;

    -- Communication signals (between master and slave DUTs)
    signal sclk_s : std_logic;
    signal mosi_s : std_logic;
    signal miso_s : std_logic;
    signal n_ss_s : std_logic;

begin

    -- A vous de jouer !

    -- Simulation monitor
    monitor : simulation_monitor
    generic map (
        drain_time => 2000 ns, -- Timeout (objections)
        beat_time => 2000 ns, -- Timeout (heart beat)
        final_reporting => rep
    );

    -- Clock generation
    clock_generator(clk_master_sti, MASTER_CLK_PERIOD);
    clock_generator(clk_slave_sti, SLAVE_CLK_PERIOD);

    -- Reset generation
    simple_startup_reset(rst_master_sti, MASTER_CLK_PERIOD * 10);
    simple_startup_reset(rst_slave_sti, SLAVE_CLK_PERIOD * 10);

    -- Devices Under Test
    DUT_Master : tictactoe_top
        port map(
            clk_i        => clk_master_sti,
            rst_i        => rst_master_sti,
            keys_i       => keys_master_sti,
            switchs_i    => switches_master_sti,
            game_state_o => game_state_master_obs,
            master_en_i  => '1', -- Master
            -- Master Interface
            sclk_o       => sclk_s,
            mosi_o       => mosi_s,
            miso_i       => miso_s,
            n_ss_o       => n_ss_s,
            -- Slave Interface (is not used since this is a master)
            sclk_i       => '0',
            mosi_i       => '0',
            miso_o       => open,
            n_ss_i       => '0'
        );

    DUT_Slave : tictactoe_top
        port map(
            clk_i        => clk_slave_sti,
            rst_i        => rst_slave_sti,
            keys_i       => keys_slave_sti,
            switchs_i    => switches_slave_sti,
            game_state_o => game_state_slave_obs,
            master_en_i  => '0', -- Slave
            -- Master Interface (is not used since this is a slave)
            sclk_o       => open,
            mosi_o       => open,
            miso_i       => '0',
            n_ss_o       => open,
            -- Slave Interface
            sclk_i       => sclk_s,
            mosi_i       => mosi_s,
            miso_o       => miso_s,
            n_ss_i       => n_ss_s
        );

end architecture;
