-------------------------------------------------------------------------------
-- HEIG-VD, Haute Ecole d'Ingenierie et de Gestion du canton de Vaud
-- Institut REDS, Reconfigurable & Embedded Digital Systems
--
-- File         : tictactoe_tb.vhd
-- Description  :
--
-- Author       :
-- Date         : 30.05.17
-- Version      : 0.0
--
-- Dependencies :
--
--| Modifications |------------------------------------------------------------
-- Version   Author Date               Description
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library tlmvm;
context tlmvm.tlmvm_context;

use work.tictactoe_config_pkg.all;

entity tictactoe_tb is
end entity;

architecture testbench of tictactoe_tb is

    ---------------
    -- Constants --
    ---------------
    constant CLK_PERIOD : time := 10 ns;

    ----------------
    -- Components --
    ----------------
    component tictactoe_top is
        port (
            clk_i        : in  std_logic;
            rst_i        : in  std_logic;
            -- Game controls
            keys_i       : in  std_logic_vector(2 downto 0);
            switchs_i    : in  std_logic_vector(7 downto 0);
            -- Game state
            game_state_o : out game_state_t;
            -- Master interface
            master_en_i  : in  std_logic;
            sclk_o       : out std_logic;
            mosi_o       : out std_logic;
            miso_i       : in  std_logic;
            n_ss_o       : out std_logic;
            -- Slave interface
            sclk_i       : in  std_logic;
            mosi_i       : in  std_logic;
            miso_o       : out std_logic;
            n_ss_i       : in  std_logic);
    end component;

    ----------------
    -- Procedures --
    ----------------
    procedure rep(status : finish_status_t) is
    begin
        report "End of simulation";
    end rep;

    -------------
    -- Signals --
    -------------
    signal finished_s     : std_logic := '0';
    signal clk_sti        : std_logic;
    signal rst_sti        : std_logic;
    signal keys_sti       : std_logic_vector(2 downto 0);
    signal switches_sti   : std_logic_vector(7 downto 0);
    signal game_state_obs : game_state_t;
    signal master_sti     : std_logic;
    signal sclk_obs       : std_logic;
    signal mosi_obs       : std_logic;
    signal miso_sti       : std_logic;
    signal n_ss_obs       : std_logic;
    signal sclk_sti       : std_logic;
    signal mosi_sti       : std_logic;
    signal miso_obs       : std_logic;
    signal n_ss_sti       : std_logic;

begin

    -- A vous de jouer !

    -- Simulation monitor
    monitor : simulation_monitor
    generic map (
        drain_time => 2000 ns, -- Timeout (objections)
        beat_time => 2000 ns, -- Timeout (heart beat)
        final_reporting => rep
    );

    -- Clock generation
    clock_generator(clk_sti, CLK_PERIOD);

    -- Reset generation
    simple_startup_reset(rst_sti, CLK_PERIOD * 10);

    -- Device Under Test
    DUT : tictactoe_top
        port map(
            clk_i        => clk_sti,
            rst_i        => rst_sti,
            keys_i       => keys_sti,
            switchs_i    => switches_sti,
            game_state_o => game_state_obs,
            master_en_i  => master_sti,
            sclk_o       => sclk_obs,
            mosi_o       => mosi_obs,
            miso_i       => miso_sti,
            n_ss_o       => n_ss_obs,
            sclk_i       => sclk_sti,
            mosi_i       => mosi_sti,
            miso_o       => miso_obs,
            n_ss_i       => n_ss_sti
        );

end architecture;
