------------------------------------------------------------------------------------------
-- HEIG-VD ///////////////////////////////////////////////////////////////////////////////
-- Haute Ecole d'Ingenerie et de Gestion du Canton de Vaud
-- School of Business and Engineering in Canton de Vaud
------------------------------------------------------------------------------------------
-- REDS Institute ////////////////////////////////////////////////////////////////////////
-- Reconfigurable Embedded Digital Systems
------------------------------------------------------------------------------------------
--
-- File                 : tictactoe_config_pkg.vhd
-- Author               : Flavio Capitao (FCC)
-- Date                 : 26.04.2017
--
-- Context              : Tic-Tac-Toe game
--
------------------------------------------------------------------------------------------
-- Description : Constants, configuration and types for Tic-Tac-Toe game
--   
------------------------------------------------------------------------------------------
-- Dependencies : -
--   
------------------------------------------------------------------------------------------
-- Modifications :
-- Ver    Date        Engineer      Comments
-- 0.0    See header  FCC           Initial version.
--
------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
 
package tictactoe_config_pkg is
 

    ------------------------
    --   Global constants
    ------------------------
    constant CLK_FREQ    : integer := 27; -- MHz

    ------------------------
    --   Global types
    ------------------------

    -- Game state record
    type game_state_t is record
        -------------------
        -- Enabled marks :
        player1_reg : std_logic_vector(8 downto 0); -- player 1 
        player2_reg : std_logic_vector(8 downto 0); -- player 1
        sel_reg     : std_logic_vector(8 downto 0); -- pointer/selector
        -- bits encoding :
        --   0 | 1 | 2
        --  -----------
        --   3 | 4 | 5
        --  -----------
        --   6 | 7 | 8

        --------------------
        -- Current player :
        curr_player : std_logic;
        -- '0': player 1
        -- '1': player 2
        
        ---------------
        -- Win/draw :
        curr_player_win : std_logic;
        draw_match     : std_logic;

        --------------
        -- Win line :
        win_line : std_logic_vector(7 downto 0);
        -- bit 0 : mark(0,1,2) -> row 1  
        -- bit 1 : mark(3,4,5) -> row 2  
        -- bit 2 : mark(6,7,8) -> row 3  
        -- bit 3 : mark(0,3,6) -> col 1  
        -- bit 4 : mark(1,4,7) -> col 2  
        -- bit 5 : mark(2,5,8) -> col 3  
        -- bit 6 : mark(0,4,8) -> diag \ 
        -- bit 7 : mark(2,4,6) -> diag / 
    end record game_state_t;  


   
end package tictactoe_config_pkg;
